#!/usr/bin/python3
"""Run fileman gRPC server.

Usage: fred-fileman-server [options]
       fred-fileman-server -h | --help
       fred-fileman-server --version

Options:
  -h, --help                show this help message and exit
  --version                 show program's version number and exit
  --config=FILE             set custom config file
  -p, --port=PORT           set custom port
  --connection=CONNECTION   set custom database connection string
  --storage=STORAGE         set custom storage root
"""
from concurrent.futures import ThreadPoolExecutor
from time import sleep
from typing import Iterable

import grpc
from docopt import docopt
from fred_api.fileman.service_fileman_grpc_pb2_grpc import add_FileServicer_to_server
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

import fileman
from fileman.service import FileServicer
from fileman.settings import SETTINGS


def main(argv: Iterable[str] = None):
    """Start the server."""
    options = docopt(__doc__, argv=argv, version=fileman.__version__)
    SETTINGS.load(options['--config'])
    SETTINGS.set_options(options)

    server = grpc.server(ThreadPoolExecutor())
    engine = create_engine(SETTINGS['db_connection'])
    add_FileServicer_to_server(FileServicer(SETTINGS['storage_root'], sessionmaker(bind=engine)), server)
    server.add_insecure_port('[::]:{port}'.format(port=SETTINGS['grpc_port']))
    server.start()
    # Sadly this is the recommended way to keep the server running.
    try:
        while True:
            sleep(86400)
    except KeyboardInterrupt:
        server.stop(0)


if __name__ == '__main__':
    main()
