from concurrent.futures import ThreadPoolExecutor
from unittest import TestCase
from unittest.mock import call, patch

from grpc._utilities import DictionaryGenericHandler
from testfixtures import Comparison

from fileman.commands import server

from .utils import override_settings


@override_settings(db_connection='sqlite:///:memory:')
class ServiceMainTest(TestCase):
    def test_main(self):
        with patch('fileman.commands.server.sleep', side_effect=KeyboardInterrupt):
            with patch('fileman.commands.server.grpc') as grpc_mock:
                server.main([])

        calls = [call.server(Comparison(ThreadPoolExecutor)),
                 call.server().add_generic_rpc_handlers((Comparison(DictionaryGenericHandler), )),
                 call.server().add_insecure_port('[::]:50051'),
                 call.server().start(),
                 call.server().stop(0)]
        self.assertEqual(grpc_mock.mock_calls, calls)
